(function () {
    let todos = null;
    const appLoadingEle = document.getElementById("app-loading-display");
    const todoWrapperEle = document.getElementById("todo-display");
    const todoEmptyTextEle = document.getElementById("todo-empty-text");
    const todoTableEle = document.getElementById("todo-table");
    const todoTableBodyEle = document.querySelector("#todo-table tbody");
    const addTodoEle = document.getElementById("add-todo-wrapper");
    const addTodoForm = document.getElementById("add-todo-form");

    const todoTitleField = document.getElementById("todo-title");

    function fetchTodos() {
        fetch("http://localhost:8080/todos")
            .then(r => r.json())
            .then(r => todos = r)
            .then(renderTodos)
            .catch((e) => {
                init();
            });
    }

    function init() {
        fetch("http://localhost:8080/init")
            .then(() => fetchTodos())
            .catch((e) => displayError(e));
    }

    function renderTodos() {
        appLoadingEle.classList.add("d-none");
        todoWrapperEle.classList.remove("d-none");
        addTodoEle.classList.remove("d-none");

        if (todos.length === 0) {
            todoEmptyTextEle.classList.remove("d-none");
            todoTableEle.classList.add("d-none");
            return;
        }

        todoEmptyTextEle.classList.add("d-none");
        todoTableEle.classList.remove("d-none");

        while (todoTableBodyEle.firstChild) {
            todoTableBodyEle.removeChild(todoTableBodyEle.firstChild);
        }

        todos.forEach((todo) => {
            const todoId = todo.id;

            const row = document.createElement("tr");

            row.appendChild(createCell(todoId));
            row.appendChild(createCell(todo.title));
            row.appendChild(createCheckboxCell(todo));

            const actionCell = document.createElement("td");

            const deleteButton = document.createElement("button");
            deleteButton.classList.add(...["btn", "btn-sm", "btn-danger"]);
            deleteButton.innerText = "Delete";

            deleteButton.addEventListener("click", (e) => {
                e.preventDefault();
                deleteTodo(todoId);
            });

            actionCell.appendChild(deleteButton);

            row.appendChild(actionCell);

            todoTableBodyEle.appendChild(row);
        });
    }

    function createCell(contents) {
        const cell = document.createElement("td");
        cell.innerText = contents;
        return cell;
    }

    function createCheckboxCell(todo) {
        const cell = document.createElement("td");
        const checkbox = document.createElement("input");
        checkbox.type = "checkbox";
        checkbox.checked = todo.completed;
        // on click
        checkbox.addEventListener("click", (e) => {
            e.preventDefault();
            updateTodoStatus(todo.id, todo.title, !todo.completed);
        });
        cell.appendChild(checkbox);
        return cell;
    }

    function deleteTodo(todoId) {
        fetch(`http://localhost:8080/delete_todo?id=${todoId}`)
            .then(() => fetchTodos());
    }

    function displayError(err) {
        alert("Error:" + err);
    }

    function onAddFormSubmit(e) {
        e.preventDefault();

        const data = {
            title: todoTitleField.value,
            completed: false
        };

        fetch("http://localhost:8080/create_todo", {
            method: "POST",
            body: JSON.stringify(data),
            headers: {"Content-type": "application/json"},
        }).then(() => fetchTodos())
            .then(() => resetAddTodoForm());

        alert("Todo added");
    }

    function updateTodoStatus(todoId, title, completed) {
        fetch("http://localhost:8080/update_todo", {
            method: "POST",
            body: JSON.stringify({
                id: todoId,
                title: title,
                completed: completed
            }),
            headers: {"Content-type": "application/json"},
        })
            .then(() => fetchTodos());
    }

    function resetAddTodoForm() {
        todoTitleField.value = "";
    }

    fetchTodos();
    addTodoForm.addEventListener("submit", onAddFormSubmit);
})();